#!/bin/bash
trap "exit 0" 2

ACTIVY_DISP_ERR=0
######################################
# Installationsvorgaben    
######################################

# Installations-Platte
HD_DEV="/dev/hda"

# Minimal erlaubte Groesse der Platte
DISK_MINSIZE=5000

# Partitionsnummern ( Falls BOOT=0 wird keine separate BootPartition angelegt -> Dual-Boot)
BOOT_PART_NUM=1
SYSTEM_PART_NUM=2
SWAP_PART_NUM=3
VIDEO_PART_NUM=5

# Minimal erlaubte Partitionsgroesse
BOOT_PART_MINSIZE=15
SYSTEM_PART_MINSIZE=3700
SWAP_PART_MINSIZE=64
VIDEO_PART_MINSIZE=1000

# Standard Partitionsgroesse
BOOT_PART_DEFSIZE=50
SYSTEM_PART_DEFSIZE=6000
SWAP_PART_DEFSIZE=260
VIDEO_PART_DEFSIZE=10

# Standard Filesystem ( ext3,xfs,reiserfs )
FS_ROOT_DEFAULT="ext3"
FS_VIDEO_DEFAULT="xfs"

# Sollen die vorhandenen Einstellungen des alten Systems uebernommen werden ?
DO_BACKUP=1
BACKUP_DONE=0

########################################
### ab hier nichts mehr aendern
########################################

FS_ROOT="auto"
FS_VIDEO=""

# Log-Datei
LOG_FILE="/install.log"
PARTED_OUT="/tmp/parted.out"
EXEC_RC=0

INST_DISK=""
ROOT_DISK=""

ALL_PARTS=""
AUTO_PART_DONE=0

#fuer die Optik
NUM_PACKAGES=0
ACT_PACKAGE=0
CHOOSEN_PARTS=" "
PARTITION_VALID=0

log() {
   echo "$(date +"%y%m%d %T") <$*>" >> $LOG_FILE
}

showdisp() {
   if [ $ACTIVY_DISP_ERR -lt 3 ] ; then
      if [ $(afp-tool -a 0 -t "$1" 2>&1 | grep -c "Failing Command") -gt 0 ] ; then
         ACTIVY_DISP_ERR=$(($ACTIVY_DISP_ERR + 1))
      fi
   fi      	 
}

blinkdisp() {   
   afp-tool -R $1 -B $1 >/dev/null 2>&1
}

msgKey() {
   if [ "$MP_AUTO_INST" = "0" ] ; then
      APP_STR=" - Enter"
   else
      APP_STR=""
   fi
   echo ""
   echo -e "\007"
   echo -e "\033[44;37m$1$APP_STR\033[0m"
   log $*
   if [ "$MP_AUTO_INST" != "1" ] ; then
      read xx
   else
      sleep 2
   fi
}

msgInfo() {
   echo ""
   echo -e "\007"
   echo -e "\033[44;37m$1 ...\033[0m"
   log $*
   sleep 2
}


execCmd() {
   if [ "$2" = "" ] ; then
      log $1
      echo "Exec <$1>"
      $1 2>&1 | tee -a $LOG_FILE
   else
      log "$1 | $2"
      echo "Exec <$1 | $2>"
      $1 | $2 2>&1 | tee -a $LOG_FILE
   fi      
   EXEC_RC=$?
   log "RC: $EXEC_RC"
   [ "$EXEC_RC" != "0" ] && echo "RC <$EXEC_RC>"
   sleep 1
}


unpackArchive() {
   ACT_PACKAGE=$(($ACT_PACKAGE+1))
   msgInfo "Entpacke $1 (Paket $ACT_PACKAGE von $NUM_PACKAGES)"
   showdisp "Entpacke|Paket $ACT_PACKAGE von $NUM_PACKAGES"
   BL="/mnt/mnt/data/$(basename $1).log"
   [ -f $BL ] && rm -f $BL
   7zr x -so $1 2>>$BL | tar -C /mnt -xf - 2>&1 >>$BL
   cat "$BL" | tee -a $LOG_FILE
   if [ "$(grep "Everything is Ok" "$BL")" = "" ] ; then
      echo "Installation abgebrochen"
      log "Installation abgebrochen"
      showdisp "Installation|abgebrochen"
      exit
   fi
   sleep 1
}


autoPart() {
   clear
   echo ""
   echo -e "\033[44;37mCtrl + C bricht ab !\033[0m"
   echo ""
   
   i=10
   blinkdisp 1
   while [ $i != 0 ] ; do
      showdisp "Formatierung|in $i sec"
      echo -e "\033[5;47;31mSaemtliche Partitionen auf $HD_DEV werden in $i Sekunde(n) neu angelegt !!!\033[0m"
      echo -e "\007"
      sleep 1
      i=`expr $i - 1`
   done
   blinkdisp 0
   
   showdisp "Partitionen|werden angelegt"
   execCmd "dd if=/dev/zero of=$HD_DEV bs=512 count=1"
   execCmd "parted -s $HD_DEV mklabel msdos"
   
   parted --script $HD_DEV print > $PARTED_OUT
   execCmd "cat $PARTED_OUT"
   
   execCmd "parted -s $HD_DEV mkpart primary ext2 0 $BOOT_PART_DEFSIZE"
   P_END=$(($BOOT_PART_DEFSIZE + $SYSTEM_PART_DEFSIZE))
   execCmd "parted -s $HD_DEV mkpart primary ext2 $BOOT_PART_DEFSIZE $P_END"
   P_START=$P_END
   P_END=$(($P_START + $SWAP_PART_DEFSIZE))
   execCmd "parted -s $HD_DEV mkpart primary linux-swap $P_START $P_END"
   DSIZE=`cat $PARTED_OUT | grep geometry | sed 's/.*-//' | sed 's/ .*//'`
   execCmd "parted -s $HD_DEV mkpart extended $P_END $DSIZE"
   if [ ! "`tail -n 1 $LOG_FILE | grep \"^Error\"`" = "" ] ; then
      DS1=`echo "$DSIZE" | cut -d '.' -f1`
      DS2=`echo "$DSIZE" | cut -d '.' -f2`
      if [ "$DS2" = "000" ] ; then
         DS2=999
         DS1=$(($DS1 - 1))
      else
         DS2=$(($DS2 - 1))
      fi
   
      execCmd "parted -s $HD_DEV mkpart extended $P_END $DS1.$DS2"
      if [ ! "`tail -n 1 $LOG_FILE | grep \"^Error\"`" = "" ] ; then
         DS1=$(($DS1 - 1))
         execCmd "parted -s $HD_DEV mkpart extended $P_END $DS1.$DS2"
      fi
      DSIZE="$DS1.$DS2"
   fi
   execCmd "parted -s $HD_DEV mkpart logical ext2 $P_END $DSIZE"
   
   msgInfo "Partitionen wurden angelegt"
   parted --script $HD_DEV print > $PARTED_OUT
   
   export AUTO_PART_DONE=1
}


checkPart() {
   partNum=$1; partSize=$2; partName=$3
   if [ $1 -eq 0 ] ; then
      msgKey "$partName - Ungueltige PartNum $partNum"
      return 0
   fi
   cpRc=1
   field=`grep ^$partNum $PARTED_OUT`
   start=`echo $field | cut -d ' ' -f2 | cut -d '.' -f1`
   if [ $start ]; then
      end=`echo $field | cut -d ' ' -f3 | cut -d '.' -f1`
      fstype=`echo $field | cut -d ' ' -f5`
      psize=`expr $end - $start`
      echo "$partName: $HD_DEV$partNum: $psize MB - $fstype"
      if [ $partSize -gt $psize ]; then
         msgKey  "$partName $HD_DEV$partNum zu klein - Minimum $partSize MB"
         cpRc=0
      fi
   else
      msgKey "$partName $HD_DEV$partNum nicht vorhanden"
      cpRc=0
   fi
   
   return $cpRc
}

choosePartition() {
   rm -rf /tmp/ip.* > /dev/null 2>&1
   NP=0
   CPN=0
   echo "dialog --backtitle \"VDR Installationsprogramm\" --title \"$1-Partition waehlen\" --menu \\" > /tmp/ip.tmp
   if [ "$1" = "Boot" ] ; then
      echo "\"Boot und SystemPartition koennen dieselbe sein" >> /tmp/ip.tmp
   else
      echo "\"" >> /tmp/ip.tmp
   fi
   [ -f /tmp/ip.tmp1 ] && rm /tmp/ip.tmp1
   for i in $ALL_PARTS ; do
      PN=$(echo $i | cut -f 1 -d "_")
      PS=$(echo $i | cut -f 2 -d "_")
      if [ $PS -ge $2 ] ; then
         if [ "$1" = "Boot" ] && [ $PN -ne $SYSTEM_PART_NUM ] && [ $PN -ne 1 ] ; then
            continue;
         fi
         if [ "$1" != "Boot" ] && [ "$(echo " $CHOOSEN_PARTS " | grep " $PN ")" != "" ] ; then
            continue
         fi
         CPN=$PN
         NP=$(($NP + 1))
         printf "\"${INST_DISK}${PN}\" \"%8d MB\" " $PS >> /tmp/ip.tmp1
      fi
   done
   [ $NP -lt 1 ] && return $CPN
   echo "Mindestgroesse fuer die $1-Partition: $2 MB\" $(($NP + 8)) 58 $NP \\" >> /tmp/ip.tmp
   cat /tmp/ip.tmp1 >> /tmp/ip.tmp
   echo " 2>/tmp/menu.tmp" >>/tmp/ip.tmp
   echo "if [ \"\$?\" != \"0\" ] ; then" >>/tmp/ip.tmp
   echo "   rm -f /tmp/menu.tmp" >>/tmp/ip.tmp
   echo "fi" >>/tmp/ip.tmp

   clear
   rm -f /tmp/menu.tmp
   bash /tmp/ip.tmp
    
   if [ -s /tmp/menu.tmp ] ; then
      PN=$(cat /tmp/menu.tmp | sed -e "s/$INST_DISK//")
      CHOOSEN_PARTS="${CHOOSEN_PARTS} ${PN}"
      return $PN
   fi
   return 0
}

getAllParts() {
   AP=$(cat $PARTED_OUT | grep "^[1-9]" | grep -v " extended " | cut -f 1 -d " ")
   ALL_PARTS=""
   for i in $AP ; do
      field=$(grep "^$i " $PARTED_OUT | tr -s " ")
      start=$(echo $field | cut -d ' ' -f2 | cut -d '.' -f1)
      if [ $start ]; then
         end=$(echo $field | cut -d ' ' -f3 | cut -d '.' -f1)
         size=$(($end - $start))
         ALL_PARTS="$ALL_PARTS ${i}_${size}"
      fi
   done 
   CHOOSEN_PARTS=" "
   choosePartition "System" $SYSTEM_PART_MINSIZE
   SYSTEM_PART_NUM=$?
   choosePartition "Boot" $BOOT_PART_MINSIZE
   BOOT_PART_NUM=$?
   choosePartition "Swap" $SWAP_PART_MINSIZE
   SWAP_PART_NUM=$?
   choosePartition "Video" $VIDEO_PART_MINSIZE
   VIDEO_PART_NUM=$?
   PARTITION_VALID=0
   if checkAllParts ; then
      dialog --defaultno --yesno "Partitionierung ungueltig - Abbrechen ?" 5 45
      return $?
   else
      PARTITION_VALID=1
      MENU_TEXT=""
      for i in $ALL_PARTS ; do
         PN=$(echo $i | cut -f 1 -d "_")
         PS=$(echo $i | cut -f 2 -d "_")
         if [ $PN -eq $BOOT_PART_NUM ] ; then
            if [ $PN -eq $SYSTEM_PART_NUM ] ; then
               PNAME="- Boot + System"
            else
               PNAME="- Boot"
            fi
         elif [ $PN -eq $SYSTEM_PART_NUM ] ; then
            PNAME="- System"
         elif [ $PN -eq $SWAP_PART_NUM ] ; then
            PNAME="- Swap"
         elif [ $PN -eq $VIDEO_PART_NUM ] ; then
            PNAME="- Video"
         else
            PNAME=""
         fi
         MENU_TEXT="$MENU_TEXT\\n${INST_DISK}${PN} $(printf %8d $PS) $PNAME"
      done
      dialog --title "   Partitionierung uebernehmen   " --yesno "$MENU_TEXT" 0 0
      return $?
   fi
   #echo "$SYS - $BOOT - $VID - $SWAP"
   #echo "$ALL_PARTS"
}

checkAllParts() {
   clear
   capRc=1
   
   #partNum=1; partSize=10; partSWP=0
   if checkPart $BOOT_PART_NUM $BOOT_PART_MINSIZE "Boot" ; then
      capRc=0
   fi
   
   #partNum=2; partSize=1500; partSWP=0
   if checkPart $SYSTEM_PART_NUM $SYSTEM_PART_MINSIZE "System" ; then
      capRc=0
   fi
   
   #partNum=3; partSize=120; partSWP=1
   if checkPart $SWAP_PART_NUM $SWAP_PART_MINSIZE "Swap" ; then
      capRc=0
   fi
   
   #partNum=5; partSize=100; partSWP=0
   if checkPart $VIDEO_PART_NUM $VIDEO_PART_MINSIZE "Video" ; then
      capRc=0
   fi
   
   DSIZE=`cat $PARTED_OUT | grep geometry | sed 's/.*-//' | sed 's/ .*//'`
   echo "DiskSize: $DSIZE MB"
   
   return $capRc
}

findDisks() {
   rm -rf /tmp/ip.* > /dev/null 2>&1
   echo "dialog --backtitle 'VDR Installationsprogramm' --title 'Installations-Platte auswaehlen' \\" > /tmp/ip.tmp
   FOUND_DISKS="$(grep -H 0 /sys/block/???/removable | cut -f 4 -d "/")"
   ALL_DISKS=""
   COUNT_DISKS=0
   for i in $FOUND_DISKS ; do
      DS=$(cat /sys/block/${i}/size)
      DEXP=${#DS}
      if [ $DEXP -lt 10 ] ; then
         DISK_SIZE=$((${DS}/2000))
      else
         IDX=$(($DEXP-3))
         DISK_SIZE=$((${DS:0:${IDX}}/2))
      fi	 
      if [ $DISK_SIZE -ge $DISK_MINSIZE ] ; then
         if [ "$ALL_DISKS" = "" ] ; then
            ALL_DISKS="${i}"
            INST_DISK="${i}"
         else
            ALL_DISKS="$ALL_DISKS ${i}"
         fi
         echo "\"$i\" \"$DISK_SIZE MB\" \\" >>/tmp/ip.tm1
         COUNT_DISKS=$(($COUNT_DISKS + 1))
      else
         msgKey "Disk ${i} ist zu klein ($DISK_SIZE < $DISK_MINSIZE)"
      fi
   done
   ROOT_DISK=$INST_DISK
   if [ $COUNT_DISKS -ge 2 ] && [ "$MP_AUTO_INST" = "0" ] ; then
      LN=$(($COUNT_DISKS + 6))

      echo "Disks: < $ALL_DISKS >"
      echo "  --menu '' $LN 40 $COUNT_DISKS \\" >> /tmp/ip.tmp
      cat /tmp/ip.tm1 >> /tmp/ip.tmp
      echo " 2>/tmp/menu.tmp" >>/tmp/ip.tmp
      echo "if [ \"\$?\" != \"0\" ] ; then" >>/tmp/ip.tmp
      echo "   rm -f /tmp/menu.tmp" >>/tmp/ip.tmp
      echo "fi" >>/tmp/ip.tmp

      rm -f /tmp/menu.tmp
      bash /tmp/ip.tmp
    
      if [ -s /tmp/menu.tmp ] ; then
         INST_DISK=`cat /tmp/menu.tmp`
      fi
   fi
   if [ "$INST_DISK" = "" ] ; then
      return 0
   fi
   HD_DEV="/dev/$INST_DISK"
   return 1
}


checkPrereq() {
   if [ "$(grep "^flags" /proc/cpuinfo | grep " mmx" | grep " sse")" = "" ] ; then
      dialog --defaultno --yesno "CPU wird nicht unterstuetzt - Installation abbrechen ?" 5 60
      return $?
   fi
   return 1
}


doPart() {
   parted --script $HD_DEV print > $PARTED_OUT
   if [ "$MP_AUTO_INST" = "1" ] ; then
      if checkAllParts ; then
         autoPart
         if checkAllParts ; then
            msgKey "Partitionierung ungueltig"
            return 0
         fi
      fi
      return 1
   fi
   
   while [ 0 ] ; do
      clear
      dialog --backtitle "VDR Installationsprogramm" --title "PARTITIONS MENU" --menu \
      "AUTO  - macht die Standard-Partitionierung (empfohlen)
KEEP  - belaesst die Partitionierung (Videos bleiben erhalten)
CFDSK - Fuer Experten ( und fuer DualBoot )

Es werden auf $HD_DEV folgende Partitionen benoetigt:
 1: primaer /boot  Type83 >= $BOOT_PART_MINSIZE MB ( Default: $BOOT_PART_DEFSIZE )
 2: primaer /      Type83 >= $SYSTEM_PART_MINSIZE MB ( $SYSTEM_PART_DEFSIZE )
 3: primaer /swap  Type82 >= $SWAP_PART_MINSIZE MB ( $SWAP_PART_DEFSIZE )
 5: logisch /video Type83 >= $VIDEO_PART_MINSIZE MB ( Restliche Platte )" \
       19 68 4 \
      "AUTO" "Automatische Partitionierung mit Standard-Groessen" \
      "KEEP" "Partitionierung belassen" \
      "CFDSK" "CFDISK fuer die manuelle Partitionierung starten" \
      "EXIT" "Installation beenden" 2> /tmp/menu.tmp
      
      case $? in
         0)
           ;;
         1)
           return 0
           ;;
         255)
           return 0
           ;;
      esac
      clear
      
      menu=`cat /tmp/menu.tmp`
      rm -f /tmp/menu.tmp
      
      if [ "$menu" = "AUTO" ]; then
         autoPart
      elif [ "$menu" = "CFDSK" ]; then
         cfdisk $HD_DEV
      elif [ "$menu" = "EXIT" ]; then
         return 0
      fi
      parted --script $HD_DEV print > $PARTED_OUT
      
      if getAllParts ; then
         return $PARTITION_VALID
      fi
   done
}


doFormat() {
   if [ "$AUTO_PART_DONE" = "0" ] && [ "$DO_BACKUP" = "1" ] ; then
      execCmd "mount $HD_DEV$SYSTEM_PART_NUM /mnt"
      if [ -d /mnt/etc ] ; then
         mkdir /bkp
         execCmd "mount $HD_DEV$VIDEO_PART_NUM /bkp"
         if [ -d /bkp/video ] ; then
            if [ "$MP_AUTO_INST" != "1" ] ; then
               dialog --yesno "Soll die alte Konfiguration gesichert werden ?" 5 55
               if [ $? != 0 ] ; then
                  DO_BACKUP=0
               fi
            fi
            if [ "$DO_BACKUP" = "1" ] ; then
               showdisp "Konfiguration|wird gesichert"
               if [ -d /bkp/g2v-backup ] ; then
                  execCmd "rm -rf /bkp/g2v-backup/*"
               else
                  mkdir /bkp/g2v-backup
               fi
               msgInfo "Sichere /etc /_config /home und /root nach /g2v-backup auf $HD_DEV$VIDEO_PART_NUM"
	       
               if [ ! -d /bkp/g2v-backup/etc ] ; then
                  execCmd "cp -af /mnt/etc /bkp/g2v-backup"
                  BACKUP_DONE=1		  
               fi
               if [ ! -d /bkp/g2v-backup/_config ] ; then
                  execCmd "cp -af /mnt/_config /bkp/g2v-backup"
                  BACKUP_DONE=1
               fi
               if [ ! -d /bkp/g2v-backup/home ] ; then
                  execCmd "cp -af /mnt/home /bkp/g2v-backup"
                  BACKUP_DONE=1		  
               fi
               if [ ! -d /bkp/g2v-backup/root ] ; then
                  execCmd "cp -af /mnt/root /bkp/g2v-backup"
                  BACKUP_DONE=1		  
               fi
            fi
         fi
         execCmd "umount $HD_DEV$VIDEO_PART_NUM"
      fi   
      execCmd "umount $HD_DEV$SYSTEM_PART_NUM"
   fi
   
   
   if [ "$BOOT_PART_NUM" != "$SYSTEM_PART_NUM" ] ; then
      showdisp "Formatiere|Boot Partition"
      msgInfo "Formatiere Boot-Partition $HD_DEV$BOOT_PART_NUM mit ext2"
      execCmd "mke2fs $HD_DEV$BOOT_PART_NUM"
      execCmd "tune2fs -i 0 -c 0 $HD_DEV$BOOT_PART_NUM"
      sleep 3
   fi
   
   
   if [ "$MP_AUTO_INST" != "1" ] ; then
      clear
      dialog --backtitle "VDR Installationsprogramm" --title "System-Partition formatieren" --menu \
      "Waehle das Datei-System fuer die System-Partition $HD_DEV$SYSTEM_PART_NUM" \
      11 65 3 \
      "EXT3" "Sehr solide ( empfohlen )" \
      "XFS" "" \
      "REISERFS" "" 2> /tmp/menu.tmp
   
      case $? in
         0)
           ;;
         1)
           return 0
           ;;
         255)
           return 0
           ;;
      esac
      clear
      menu=`cat /tmp/menu.tmp`
      rm -f /tmp/menu.tmp
   
      echo ""
      echo "Formatiere System-Partition $HD_DEV$SYSTEM_PART_NUM mit $menu ..."
      echo ""
      if [ "$menu" = "EXT3" ]; then
         FS_ROOT='ext3'
      elif [ "$menu" = "XFS" ]; then
         FS_ROOT='xfs'
      elif [ "$menu" = "REISERFS" ]; then
         FS_ROOT='reiserfs'
      fi
   else
      echo ""
      echo "Formatiere System-Partition $HD_DEV$SYSTEM_PART_NUM mit EXT3 ..."
      echo ""
      FS_ROOT=$FS_ROOT_DEFAULT
   fi
   
   showdisp "Formatiere|System Partition"
   if [ "$FS_ROOT" = "ext3" ]; then
      execCmd "mke2fs -j $HD_DEV$SYSTEM_PART_NUM"
      execCmd "tune2fs -i 0 -c 0 $HD_DEV$SYSTEM_PART_NUM"
   elif [ "$FS_ROOT" = "xfs" ]; then
      execCmd "mkfs.xfs -f $HD_DEV$SYSTEM_PART_NUM"
   elif [ "$FS_ROOT" = "reiserfs" ]; then
      echo "mkreiserfs -f $HD_DEV$SYSTEM_PART_NUM"
      echo y |mkreiserfs -f $HD_DEV$SYSTEM_PART_NUM
   fi
   
   msgInfo "Ernenne $HD_DEV$SWAP_PART_NUM zur Swap-Partition"
   execCmd "mkswap $HD_DEV$SWAP_PART_NUM"
   sleep 3
   
   execCmd "mount $HD_DEV$VIDEO_PART_NUM /mnt"
   FS_VIDEO=`mount | grep "$HD_DEV$VIDEO_PART_NUM" | head -n 1 | cut -f 5 -d " "`   
   umount /mnt >/dev/null 2>&1
   if [ "$FS_VIDEO" = "" ] ; then
      msgInfo "Kann video Partition nicht erkennen"
      return 0
   fi
   return 1
}


doInstall() {
   if [ "$MP_AUTO_INST" != "1" ] ; then
      APP_STR=" - Enter"
   else
      APP_STR=""
   fi
   
   umount /mnt > /dev/null 2>&1
   
   umount /mnt >/dev/null 2>&1
   
   # System und Boot partition mounten:
   if [ -f /mnt/boot ] ; then
      execCmd "umount /mnt/boot >/dev/null 2>&1"
      execCmd "umount /mnt >/dev/null 2>&1"
   fi
   execCmd "mount -t $FS_ROOT $SYSTEM_PART $HD_DEV$SYSTEM_PART_NUM /mnt"
   execCmd "mkdir /mnt/boot"
   execCmd "mkdir /mnt/mnt"
   execCmd "mkdir /mnt/mnt/data"   
   
   if [ "$BOOT_PART_NUM" != "$SYSTEM_PART_NUM" ] ; then
      execCmd "mount -t ext2 $HD_DEV$BOOT_PART_NUM /mnt/boot"
   fi
   execCmd "mount -t $FS_VIDEO $HD_DEV$VIDEO_PART_NUM /mnt/mnt/data"
   
   DT=`date +%Y%m%d`
   cp -f $INSTALL_DIR/HISTORY /mnt/HISTORY.$DT
   cp -f $INSTALL_DIR/README /mnt/README.$DT
   
   MM_DIRS="video audio pictures film games"
   SYS_DIRS="tmp log"
   DEF_DIRS="$MM_DIRS $SYS_DIRS"
   
   rm -rf /mnt/tmpfs/* > /dev/null 2>&1
   mkdir /mnt/usr
   
   for i in $DEF_DIRS ; do
      mkdir /mnt/mnt/data/${i} >/dev/null 2>&1
      rm /mnt/${i} >/dev/null 2>&1
      ln -s mnt/data/${i} /mnt/${i} >/dev/null 2>&1
   done
   for i in $MM_DIRS ; do
      rm -f /mnt/mnt/data/${i}/DISC >/dev/null 2>&1
      [ "$i" != "video" ] && ln -s /mnt/cdrom /mnt/mnt/data/${i}/DISC >/dev/null 2>&1
      rm -f /mnt/mnt/data/${i}/USB >/dev/null 2>&1
      rm -f /mnt/mnt/data/${i}/MEDIA >/dev/null 2>&1
      [ "$i" != "video" ] && ln -s /media /mnt/mnt/data/${i}/MEDIA >/dev/null 2>&1
   done
   [ -f /mnt/usr/portage ] && rm -f /mnt/usr/portage
   ln -s ../mnt/data/portage /mnt/usr/
   mkdir /mnt/mnt/data/portage > /dev/null 2>&1
   mkdir /mnt/audio/images > /dev/null 2>&1
   mkdir /mnt/audio/covers > /dev/null 2>&1
   mkdir /mnt/video/epgimages > /dev/null 2>&1
   mkdir /mnt/var > /dev/null 2>&1
   rm -rf /mnt/mnt/data/log/* > /dev/null 2>&1
   rm -rf /mnt/mnt/data/tmp/* > /dev/null 2>&1
   ln -s ../log /mnt/var/
   ln -s ../tmp /mnt/var/
   ln -s everything/current /mnt/mnt/data/log/messages
   touch /mnt/mnt/data/log/lastlog
   mkdir /mnt/mnt/data/log/mysql > /dev/null 2>&1
   mkdir /mnt/mnt/data/log/freevo >/dev/null 2>&1
   mkdir /mnt/proc >/dev/null 2>&1
   
   cp -a /dev /mnt/
   # ins gemountete Verzeichnis wechseln:
   cd /mnt
   
   if [ ! -f $INSTALL_DIR/g2v_base_usr.tar.7z ] ; then
      msgInfo "$INSTALL_DIR/g2v_base_usr.tar.7z nicht gefunden"
      return 0
   fi
   
   NUM_PACKAGES=$(($(ls $INSTALL_DIR/g2v_*.tar.7z | wc -w)))
   for i in $INSTALL_DIR/g2v_base*.tar.7z ; do
      unpackArchive $i
   done
   
   # ein bischen warten ...
   
   if [ "$(ls $INSTALL_DIR/g2v_opt*.tar.7z 2>/dev/null)" != "" ] ; then 
      for i in $INSTALL_DIR/g2v_opt*.tar.7z ; do
         unpackArchive $i
      done
   fi
   
   #fstab anpassen
   #cp -f /mnt/_config/etc/$fs1 /mnt/etc/fstab
   #cat /mnt/_config/etc/$fs2 >> /mnt/etc/fstab
   if [ "$INST_OPT" != "RESTORE" ] ; then
      msgInfo "fstab anpassen"
      rm -f /mnt/etc/fstab
      cat /mnt/_config/install/fstab | sed "\%^/dev/hda[0-9]%d" > /mnt/etc/fstab
      NLINE="${HD_DEV}${SYSTEM_PART_NUM}   /   ${FS_ROOT}   noatime 1 1"
      echo "$NLINE" >> /mnt/etc/fstab
      NLINE="${HD_DEV}${VIDEO_PART_NUM}   /mnt/data   ${FS_VIDEO}   noatime 1 2"
      echo "$NLINE" >> /mnt/etc/fstab
      NLINE="${HD_DEV}${SWAP_PART_NUM}   none   swap   sw 0 0"
      echo "$NLINE" >> /mnt/etc/fstab
      if [ "$BOOT_PART_NUM" != "$SYSTEM_PART_NUM" ] ; then
         NLINE="${HD_DEV}${BOOT_PART_NUM}   /boot   ext2   noatime 1 2"
         echo "$NLINE" >> /mnt/etc/fstab
      fi
      cp -af /mnt/_config/install/vdr/* /mnt/etc/vdr/
   fi

   rm -f /mnt/mnt/data/portage/packages/g2v.pkg /mnt/_config/update/g2v_md5_fixes > /dev/null 2>&1            
   if [ -d ${INSTALL_DIR}/g2v_inst ] ; then 
      msgInfo "Installationsdateien kopieren"     
      execCmd "cp -af ${INSTALL_DIR}/g2v_inst/* /mnt/"
      
      for i in /mnt/*.tar.bz2 ; do
         if [ -f $i ] ; then
            execCmd "tar -C /mnt -xjvf $i"
         fi	    
      done
      
      # updates entpacken
      MD5_FILE="${INSTALL_DIR}/g2v_inst/_config/update/g2v_md5_fixes"
      if [ -f $MD5_FILE ] ; then
         for i in $(grep "\.tar\.bz2" $MD5_FILE | cut -f 3 -d " ") ; do
            execCmd "tar -C /mnt -xjvf /mnt/_config/update/$i"
         done
      fi         
      rm -f /mnt/*.tar.bz2 > /dev/null 2>&1      
      
      # Check for packages to install
      if [ -e ${INSTALL_DIR}/g2v_inst/mnt/data/portage/packages/All ] ; then 
         cd ${INSTALL_DIR}/g2v_inst/mnt/data/portage/packages/All
	 ls *-*/*.tbz2 | sed -e "s/\.tbz2.*//" > /mnt/mnt/data/portage/packages/g2v.pkg 
	 cd /
      fi
   fi
   if [ -d /mnt/mnt/data/g2v-backup ] ; then
      rm -f /mnt/mnt/data/g2v-backup/.do_migrate > /dev/null 2>&1
      if [ "$BACKUP_DONE" = "1" ] && [ "$INST_OPT" != "RESTORE" ] ; then
         DO_MIGRATE=1
         if [ "$MP_AUTO_INST" != "1" ] ; then
            dialog --yesno "Soll die alte Konfiguration migriert werden ?" 0 0
            if [ $? != 0 ] ; then
	       DO_MIGRATE=0               
            fi
         fi
         [ "$DO_MIGRATE" = "1" ] && touch /mnt/mnt/data/g2v-backup/.do_migrate
      fi	 
   fi
   
   umount /mnt/mnt/data
   
   return 1
}


#
#
#
log "Gen2VDR Installation startet"
showdisp "Gen2VDR|Installation"

#disable screen blanking
echo -en "\033[9;0]"

#disable afp watchdog
afp-tool -h 0 >/dev/null 2>&1

if [ "$INSTALL_DIR" = "" ] ; then
   export INSTALL_DIR="/SETUP"
fi

if checkPrereq ; then
   echo "Installation abgebrochen"
   showdisp "Installation|abgebrochen"
   exit
fi

if [ "$MP_AUTO_INST" != "1" ] ; then
   dialog --backtitle "VDR Installationsprogramm" --title "Jetzt gehts los" --menu \
   "Wie haettest Du es denn gerne ?" \
    9 50 2 \
   "AUTO"  "Vollautomatisch von vorne bis hinten"  \
   "INTER" "Interaktiv" 2> /tmp/menu.tmp

   case $? in
      0)
        ;;
      1)
        exit
        ;;
      255)
        exit
        ;;
   esac
   clear

   menu=`cat /tmp/menu.tmp`
   rm -f /tmp/menu.tmp

   if [ "$menu" = "AUTO" ]; then
      export MP_AUTO_INST=1
   else
      export MP_AUTO_INST=0
   fi
fi


showdisp "Untersuche|Festplatte"
if findDisks ; then
   echo "Installation bei der Platten/CD Erkennung abgebrochen"
   showdisp "Installation|abgebrochen"
   exit
fi

showdisp "Partitioniere|Festplatte"
if doPart ; then
   echo "Installation bei Partitionierung abgebrochen"
   showdisp "Installation|abgebrochen"
   exit
fi

showdisp "Formatiere|Festplatte"
if doFormat ; then
   echo "Installation bei Formatierung abgebrochen"
   showdisp "Installation|abgebrochen"
   exit
fi


if doInstall ; then
   execCmd "umount /mnt >/dev/null 2>&1"
   echo "Installation beim Kopieren abgebrochen"
   showdisp "Installation|abgebrochen"
   exit
fi

showdisp "Installation wird|abgeschlossen"
cp -f $LOG_FILE /mnt/ >/dev/null 2>&1

LILOCONF="/mnt/etc/lilo.conf"
mv -f $LILOCONF $LILOCONF.org
sed -i $LILOCONF.org -e "s/lpj=[0-9]*//"
sed -i /mnt/boot/grub/grub.conf -e "s/lpj=[0-9]*//"

cat $LILOCONF.org | sed "s/^default = .*$/default = 2.6/" | \
                    sed "s%^root.*/dev/[hs]d.*%root = ${HD_DEV}${SYSTEM_PART_NUM}%" | \
                    sed "s%^boot.*%boot = /dev/$ROOT_DISK%" > $LILOCONF

ln -s /mnt/boot /
ln -s /mnt/_config /

msgInfo "Lilo ausfuehren"
execCmd "lilo -C $LILOCONF"

echo ""
echo ""
if [ "$INST_OPT" != "RESTORE" ] ; then
   echo "Installation erfolgreich beendet !"
   echo "Nach dem Reboot gehts weiter mit Stufe 2"
   sed -i /mnt/etc/inittab -e "s%bootwait:/sbin/rc boot%bootwait:/_config/install/g2v_install.sh%"
   MODCONF="/mnt/etc/modules.autoload.d/kernel-2.6"
   echo "" > $MODCONF
   #echo "natsemi" > $MODCONF
   #echo "usb-storage" >> $MODCONF
   echo "$MP_AUTO_INST" > /mnt/.autoinst
else
   echo "Restore erfolgreich beendet !"
fi
echo ""
echo "Das Passwort fuer den root-user lautet: <gen2vdr>"
msgKey "Das System wird nun neu gestartet"
showdisp "System wird|neu gestartet"
echo "Rebooting" >> /mnt/$LOG_FILE
reboot
